#version 330 core

uniform mat4 u_ModelView;
uniform mat4 u_View;
uniform mat4 u_Projection;
uniform int u_ParticleSpace;
uniform int u_ParticleOrientation;

layout(location = 0) in vec3 vIn_ms_Position;
layout(location = 1) in vec3 vIn_ss_CenterPosition;
layout(location = 2) in float vIn_Size;
layout(location = 3) in float vIn_Rotation;
layout(location = 4) in vec4 vIn_Color;
layout(location = 5) in float vIn_Lifetime;
layout(location = 6) in vec3 vIn_ss_OrientationVector;

out vec4 pIn_Color;
out vec2 pIn_ts_TexCoord;

vec4 transformToEyeSpace(vec4 vector) {
	// u_ParticleSpace = 0 -> LOCAL_SPACE
	// u_ParticleSpace = 1 -> WORLD_SPACE
	return (u_ModelView * vector * (1 - u_ParticleSpace)) + (u_View * vector * u_ParticleSpace);
}

vec4 getVertexOffset() {
	float transformed_ms_X = (cos(vIn_Rotation) * vIn_ms_Position.x - sin(vIn_Rotation) * vIn_ms_Position.y) * vIn_Size;
	float transformed_ms_Y = (sin(vIn_Rotation) * vIn_ms_Position.x + cos(vIn_Rotation) * vIn_ms_Position.y) * vIn_Size;
	
	if (u_ParticleOrientation == 0) { // u_ParticleOrientation = 0 -> SCREEN
		return vec4(transformed_ms_X, transformed_ms_Y, 0, 0);
	}
	
	else if (u_ParticleOrientation == 1) { // u_ParticleOrientation = 1 -> VELOCITY_FACE
	
		vec3 es_Velocity = transformToEyeSpace(vec4(vIn_ss_OrientationVector, 0)).xyz;
		vec3 otherCrossOperand = vec3(0, 1, 0);
		
		vec3 orientationVector = normalize(cross(es_Velocity, otherCrossOperand));
		vec3 perpendicularOrientationVector = normalize(cross(es_Velocity, orientationVector));
		
		vec4 offset = vec4(orientationVector * -transformed_ms_Y + perpendicularOrientationVector * transformed_ms_X, 0);
		
		return offset;
	}
	
	else if (u_ParticleOrientation == 2) { // u_ParticleOrientation = 2 -> VELOCITY_UP
		vec3 sightVector = -normalize(transformToEyeSpace(vec4(vIn_ss_CenterPosition, 1))).xyz; // The direction in which the eye lies
		vec3 orientationVector = normalize(transformToEyeSpace(vec4(vIn_ss_OrientationVector, 0))).xyz; // The y-axis of the particle in eye space
		vec3 perpendicularOrientationVector = normalize(cross(orientationVector, sightVector)); // The x-axis of the particle in eye space
		vec4 offset = vec4(orientationVector * transformed_ms_Y + perpendicularOrientationVector * transformed_ms_X, 0);
		return offset;
	}
	
}

vec4 getTransformedVertexPosition() {
	vec4 transformedPosition = transformToEyeSpace(vec4(vIn_ss_CenterPosition, 1));
	vec4 vertexOffset = getVertexOffset();
	return u_Projection * (transformedPosition + vertexOffset);
}

void main() {
	if (vIn_Lifetime > 0) {
		gl_Position = getTransformedVertexPosition();
		pIn_ts_TexCoord = vec2(vIn_ms_Position.x + 0.5, vIn_ms_Position.y + 0.5);
		pIn_Color = vIn_Color;
	}
	else {
		gl_Position = vec4(0, 0, 0, 0);
	}
}