#version 330 core

uniform vec3 u_es_LightDirection;

uniform sampler2D u_DiffuseAlphaMap;
uniform sampler2D u_NormalMap;
uniform int u_NormalMapUsed;
uniform sampler2D u_SpecularMap;
uniform sampler2DShadow u_ShadowMap;

uniform mat4 u_ShadowMapModelViewProjection;

uniform float u_ShadingFactor;
uniform vec3 u_AmbientCoefficient;
uniform vec3 u_DiffuseCoefficient;
uniform vec3 u_SpecularCoefficient;
uniform float u_SpecularExponent;

in vec4 pIn_ss_ShadowCoords;
in vec3 pIn_es_Position;
in vec3 pIn_es_Normal;
in vec2 pIn_ts_TexCoord;
in vec3 pIn_es_Tangent;
in vec3 pIn_es_Bitangent;
in mat3 pIn_es_TangentBitangentNormal;
in vec4 pIn_Color;

vec3 desaturate(vec3 rgb, float amount) {
    vec3 gray = vec3(dot(vec3(0.2126,0.7152,0.0722), rgb));
    return vec3(mix(rgb, gray, amount));
}

void main() {

	vec4 specularSample = texture2D(u_SpecularMap, pIn_ts_TexCoord);
	vec3 specularCoefficient = u_SpecularCoefficient * specularSample.rgb;
	vec4 diffuseAlphaSample = texture2D(u_DiffuseAlphaMap, pIn_ts_TexCoord);
	
	vec3 es_EyeDirection = normalize(-pIn_es_Position);
	
	vec3 projShadowCoords = pIn_ss_ShadowCoords.xyz / pIn_ss_ShadowCoords.w;
	vec2 uvCoords;
	uvCoords.x = 0.5 * projShadowCoords.x + 0.5;
	uvCoords.y = 0.5 * projShadowCoords.y + 0.5;
	float shadowZ = projShadowCoords.z * 0.5 + 0.5;
	
	float shadowFragmentSize = 1.0 / 1024.0;
	
	float shadowFactor = 0.0;
	
	for (int i=-1; i<=1; i++) {
	
		for (int j=-1; j<=1; j++) {
		
			vec2 offsets = vec2(j * shadowFragmentSize, i * shadowFragmentSize);
			vec3 uvc = vec3(uvCoords + offsets, shadowZ + 0.00001);
			shadowFactor += texture(u_ShadowMap, uvc);
		}
	
	}
	
	shadowFactor = (shadowFactor / 9.0);
	
	if (u_NormalMapUsed == 1) {
	
		vec4 normalSpecularSample = texture2D(u_NormalMap, pIn_ts_TexCoord);
		
		vec3 tans_LightDirection = normalize(pIn_es_TangentBitangentNormal * u_es_LightDirection);
		vec3 tans_EyeDirection = normalize(pIn_es_TangentBitangentNormal * es_EyeDirection);
		vec3 tans_Normal = normalize(normalSpecularSample.rgb * 2 - 1);
		vec3 tans_ReflectionDirection = normalize(reflect(-tans_LightDirection, tans_Normal));
		
		vec3 ambientComponent = (diffuseAlphaSample.xyz * u_AmbientCoefficient);
		ambientComponent = desaturate(ambientComponent, 1 - dot(vec3(0.2126,0.7152,0.0722), ambientComponent));
		
		float diffuseStrength = max(dot(tans_Normal, -tans_LightDirection), 0);
		
		vec3 diffuseComponent = (diffuseAlphaSample.xyz * u_DiffuseCoefficient) * diffuseStrength * shadowFactor;
		diffuseComponent = desaturate(diffuseComponent, 1-diffuseStrength);
		
		vec3 specularComponent = specularCoefficient * pow(max(dot(-tans_ReflectionDirection, tans_EyeDirection), 0), u_SpecularExponent) * shadowFactor;
	
		diffuseComponent = clamp(diffuseComponent, 0, 1);
		specularComponent = clamp(specularComponent, 0, 1);
    
		gl_FragColor = vec4( (ambientComponent + diffuseComponent + specularComponent) * (u_ShadingFactor)
							+(u_AmbientCoefficient + u_DiffuseCoefficient) * (1 - u_ShadingFactor)
							, diffuseAlphaSample.a);
	
	}
	
	else {
	
		vec3 es_Normal = normalize(pIn_es_Normal);
		vec3 es_ReflectionDirection = normalize(reflect(-u_es_LightDirection, es_Normal));
		
		vec3 ambientComponent = (diffuseAlphaSample.xyz * u_AmbientCoefficient);
		ambientComponent = desaturate(ambientComponent, dot(vec3(0.2126,0.7152,0.0722), ambientComponent));
		
		float diffuseStrength = max(dot(es_Normal, -u_es_LightDirection), 0);
		
		vec3 diffuseComponent = (diffuseAlphaSample.xyz * u_DiffuseCoefficient) * diffuseStrength * shadowFactor;
		diffuseComponent = desaturate(diffuseComponent, 1-diffuseStrength);
		
		vec3 specularComponent = specularCoefficient * pow(max(dot(-es_ReflectionDirection, es_EyeDirection), 0), u_SpecularExponent) * shadowFactor;
    
		diffuseComponent = clamp(diffuseComponent, 0, 1);
		specularComponent = clamp(specularComponent, 0, 1);
    
		gl_FragColor = vec4( (ambientComponent + diffuseComponent + specularComponent) * (u_ShadingFactor)
							+ (u_AmbientCoefficient + u_DiffuseCoefficient) * (1 - u_ShadingFactor)
							, diffuseAlphaSample.a);
							
	}
	
	gl_FragColor = gl_FragColor * pIn_Color;
	
}