#version 330 core

uniform mat4 u_ModelViewProjection;
uniform mat4 u_ModelView;
uniform mat3 u_NormalTransform;
uniform mat4 u_ShadowMapModelViewProjection;

uniform vec3 u_ms_BoneSkinPositionArray[64];
uniform vec3 u_ms_BonePositionArray[64];
uniform vec4 u_ms_BoneOrientationArray[64];

layout(location = 0) in vec3 vIn_ms_Position;
layout(location = 1) in vec3 vIn_ms_Normal;
layout(location = 2) in vec2 vIn_ts_TexCoord;
layout(location = 3) in vec3 vIn_ms_Tangent;
layout(location = 4) in vec3 vIn_ms_Bitangent;

layout(location = 5) in int vIn_BoneIndex1;
layout(location = 6) in float vIn_BoneWeight1;
layout(location = 7) in int vIn_BoneIndex2;
layout(location = 8) in float vIn_BoneWeight2;
layout(location = 9) in vec4 vIn_Color;

out vec4 pIn_ss_ShadowCoords;
out vec3 pIn_es_Position;
out vec3 pIn_es_Normal;
out vec2 pIn_ts_TexCoord;
out vec3 pIn_es_Tangent;
out vec3 pIn_es_Bitangent;
out mat3 pIn_es_TangentBitangentNormal;
out vec4 pIn_Color;

vec3 rotateVectorByQuaternion(vec4 quat, vec3 vec)
{
	return vec + 2.0 * cross( cross( vec, quat.xyz ) + quat.w * vec, quat.xyz );
}

void main() {

	pIn_ts_TexCoord = vIn_ts_TexCoord;
	
	vec3 skinnedNormal = rotateVectorByQuaternion(u_ms_BoneOrientationArray[vIn_BoneIndex1], vIn_ms_Normal) * vIn_BoneWeight1 + rotateVectorByQuaternion(u_ms_BoneOrientationArray[vIn_BoneIndex2], vIn_ms_Normal) * vIn_BoneWeight2;
	vec3 skinnedTangent = rotateVectorByQuaternion(u_ms_BoneOrientationArray[vIn_BoneIndex1], vIn_ms_Tangent) * vIn_BoneWeight1 + rotateVectorByQuaternion(u_ms_BoneOrientationArray[vIn_BoneIndex2], vIn_ms_Tangent) * vIn_BoneWeight2;
	vec3 skinnedBitangent = rotateVectorByQuaternion(u_ms_BoneOrientationArray[vIn_BoneIndex1], vIn_ms_Bitangent) * vIn_BoneWeight1 + rotateVectorByQuaternion(u_ms_BoneOrientationArray[vIn_BoneIndex2], vIn_ms_Bitangent) * vIn_BoneWeight2;
	
	pIn_es_Normal = normalize(u_NormalTransform * skinnedNormal);
	pIn_es_Tangent = normalize(u_NormalTransform * skinnedTangent);
	pIn_es_Bitangent = normalize(u_NormalTransform * skinnedBitangent);
	pIn_Color = vIn_Color;
	
	pIn_es_TangentBitangentNormal = transpose(mat3(
		pIn_es_Tangent,
		pIn_es_Bitangent,
		pIn_es_Normal
	));
	
	vec3 skinnedVertex1 = vIn_ms_Position - u_ms_BoneSkinPositionArray[vIn_BoneIndex1];
	vec3 skinnedVertex2 = vIn_ms_Position - u_ms_BoneSkinPositionArray[vIn_BoneIndex2];
	skinnedVertex1 = u_ms_BonePositionArray[vIn_BoneIndex1] + rotateVectorByQuaternion(u_ms_BoneOrientationArray[vIn_BoneIndex1], skinnedVertex1);
	skinnedVertex2 = u_ms_BonePositionArray[vIn_BoneIndex2] + rotateVectorByQuaternion(u_ms_BoneOrientationArray[vIn_BoneIndex2], skinnedVertex2);
	vec3 skinnedVertex = skinnedVertex1 * vIn_BoneWeight1 + skinnedVertex2 * vIn_BoneWeight2;
	
	pIn_ss_ShadowCoords = (u_ShadowMapModelViewProjection * vec4(skinnedVertex, 1));
	gl_Position = u_ModelViewProjection * vec4(skinnedVertex, 1);
	pIn_es_Position = (u_ModelView * vec4(vIn_ms_Position, 1)).xyz;
}